@@ -24,6 +24,7 @@ use std::{
     alloc::Layout,
     intrinsics::transmute,
     mem::{align_of, size_of, swap, MaybeUninit},
+    num::Wrapping,
     ops::{Index, IndexMut},
     ptr::NonNull,
     str::FromStr,
@@ -47,6 +48,22 @@ where
     alloc: A,
 }
 
+impl<T, A> Clone for KeyMap<T, A>
+where
+    T: Clone,
+    A: Allocator + Clone,
+{
+    fn clone(&self) -> Self {
+        let mut result = KeyMap::with_capacity(self.capacity, self.alloc.clone()).unwrap();
+
+        for (k, v) in self.iter() {
+            result.insert(k, v.clone()).unwrap();
+        }
+
+        result
+    }
+}
+
 impl<T, A: Allocator> std::fmt::Debug for KeyMap<T, A>
 where
     T: std::fmt::Debug,
@@ -103,7 +120,6 @@ impl FromStr for Handle {
 impl Handle {
     pub fn from_bytes(key: &[u8]) -> Self {
         const MASK: u64 = u32::MAX as u64;
-        // FNV-1a
         let mut hash = 2166136261u64;
         for byte in key {
             hash ^= *byte as u64;
@@ -117,27 +133,37 @@ impl Handle {
 
     pub fn from_u32(key: u32) -> Self {
         const MASK: u64 = u32::MAX as u64;
+        let key = hash_u64(key as u64, MASK);
+        Self(key)
+    }
 
-        let mut key = key as u64 + MASK * (key == 0) as u64; // to ensure non-zero keys
-        key = (((key >> 16) ^ key) * 0x45d0f3b) & MASK;
-        key = (((key >> 16) ^ key) * 0x45d0f3b) & MASK;
-        key = ((key >> 16) ^ key) & MASK;
-        debug_assert!(key != 0);
-        Self(key as u32)
+    pub fn from_u64(key: u64) -> Self {
+        const MASK: u64 = u64::MAX;
+        let key = hash_u64(key, MASK);
+        Self(key)
     }
 
     pub fn from_i64(key: i64) -> Self {
-        const MASK: u64 = u32::MAX as u64;
-
-        let mut key = key as u64 + MASK * (key == 0) as u64; // to ensure non-zero keys
-        key = (((key >> 16) ^ key) * 0x45d0f3b) & MASK;
-        key = (((key >> 16) ^ key) * 0x45d0f3b) & MASK;
-        key = ((key >> 16) ^ key) & MASK;
-        debug_assert!(key != 0);
-        Self(key as u32)
+        const MASK: u64 = u64::MAX as u64;
+        let key = hash_u64(key as u64, MASK);
+        Self(key)
     }
 }
 
+// FNV-1a
+#[inline]
+fn hash_u64(key: u64, mask: u64) -> u32 {
+    let key = key + mask * (key == 0) as u64; // to ensure non-zero keys
+
+    let mut key = Wrapping(key);
+    let mask = Wrapping(mask);
+    key = (((key >> 16) ^ key) * Wrapping(0x45d0f3b)) & mask;
+    key = (((key >> 16) ^ key) * Wrapping(0x45d0f3b)) & mask;
+    key = ((key >> 16) ^ key) & mask;
+    debug_assert!(key.0 != 0);
+    ((key >> 32) ^ key).0 as u32
+}
+
 impl From<i64> for Handle {
     fn from(key: i64) -> Self {
         Self::from_i64(key)
@@ -323,6 +349,15 @@ where
         })
     }
 
+    pub fn iter_mut(&mut self) -> impl Iterator<Item = (Handle, &'_ mut T)> + '_ {
+        let keys = self.keys.as_ptr();
+        let values = self.values.as_ptr();
+        (0..self.capacity).filter_map(move |i| unsafe {
+            let k = *keys.add(i);
+            (k.0 != 0).then(|| (k, &mut *values.add(i)))
+        })
+    }
+
     unsafe fn alloc_storage(
         alloc: &A,
         capacity: usize,
