@@ -55,7 +55,7 @@ impl<'de, T: Deserialize<'de>> Deserialize<'de> for KeyMap<T> {
 #[cfg(test)]
 mod tests {
     use crate::{
-        compiler::{CaoProgram, Lane, Module},
+        compiler::{CaoProgram, Lane},
         prelude::CaoCompiledProgram,
     };
 
@@ -105,17 +105,15 @@ mod tests {
     fn can_serialize_program_cbor() {
         let program = crate::compiler::compile(
             CaoProgram {
-                module: Module {
-                    lanes: vec![Lane {
-                        name: "main".into(),
+                lanes: [(
+                    "main".into(),
+                    Lane {
                         arguments: vec![],
                         cards: vec![],
-                    }]
-                    .into_iter()
-                    .map(|lane| (lane.name.clone(), lane))
-                    .collect(),
-                    submodules: Default::default(),
-                },
+                    },
+                )]
+                .into(),
+                submodules: Default::default(),
             },
             None,
         )
